## Task: Cosponsorship network regression: nrm and ergms
## Author: LB, GC and Georges Andres
## Date: June 2020, last updated March 2022

################################################################################
################################################################################
################################################################################

rm(list = ls())

## load data
load('input/Data_163MP.RData')

## load libraries
library(ggplot2)
#devtools::install_github("gi0na/r-ghypernet")
library(ghypernet)
library(tidyverse)
library(texreg)

## Please ensure you have the same package versions loaded before 
## attempting a full replication.
sessionInfo()
# R version 4.1.2 (2021-11-01)
# Platform: x86_64-apple-darwin17.0 (64-bit)
# Running under: macOS Catalina 10.15.7
# 
# Matrix products: default
# BLAS:   /System/Library/Frameworks/Accelerate.framework/Versions/A/Frameworks/vecLib.framework/Versions/A/libBLAS.dylib
# LAPACK: /Library/Frameworks/R.framework/Versions/4.1/Resources/lib/libRlapack.dylib
# 
# locale:
#   [1] en_US.UTF-8/en_US.UTF-8/en_US.UTF-8/C/en_US.UTF-8/en_US.UTF-8
# 
# attached base packages:
#   [1] stats     graphics  grDevices utils     datasets  methods   base     
# 
# other attached packages:
#   [1] texreg_1.37.5   forcats_0.5.1   stringr_1.4.0   dplyr_1.0.7     purrr_0.3.4     readr_2.1.1     tidyr_1.1.4     tibble_3.1.6    tidyverse_1.3.1 ghypernet_1.1.0
# [11] ggplot2_3.3.5  
# 
# loaded via a namespace (and not attached):
#   [1] Rcpp_1.0.8       cellranger_1.1.0 pillar_1.6.4     compiler_4.1.2   dbplyr_2.1.1     tools_4.1.2      lubridate_1.8.0  jsonlite_1.7.3   lifecycle_1.0.1 
# [10] gtable_0.3.0     pkgconfig_2.0.3  rlang_0.4.12     reprex_2.0.1     cli_3.1.0        rstudioapi_0.13  DBI_1.1.2        haven_2.4.3      xml2_1.3.3      
# [19] withr_2.4.3      httr_1.4.2       fs_1.5.2         generics_0.1.1   vctrs_0.3.8      hms_1.1.1        grid_4.1.2       tidyselect_1.1.1 glue_1.6.0      
# [28] R6_2.5.1         fansi_1.0.2      readxl_1.3.1     tzdb_0.2.0       modelr_0.1.8     magrittr_2.0.1   scales_1.1.1     backports_1.4.1  ellipsis_0.3.2  
# [37] rvest_1.0.2      assertthat_0.2.1 colorspace_2.0-2 utf8_1.2.2       stringi_1.7.6    munsell_0.5.0    broom_0.7.11     crayon_1.4.2 

## publicationtheme
publicationtheme <- theme(plot.title = element_text(face = "bold", size = rel(1.2), hjust = 0.5),
                          panel.background =  element_blank(), #element_rect(colour = NA),
                          plot.background = element_rect(colour = NA, fill = NA),
                          axis.title = element_text(face = "bold",size = rel(1)),
                          axis.title.y = element_text(angle=90,vjust =2),
                          axis.title.x = element_text(vjust = -0.2),
                          axis.text = element_text(), 
                          axis.line = element_line(colour="black"),
                          axis.ticks = element_line(),
                          #axis.text.y = element_blank(), 
                          #axis.ticks.y = element_blank(),
                          panel.grid.major.y = element_line(colour="#f0f0f0"),
                          panel.grid.major.x = element_line(colour="#f0f0f0"),
                          panel.grid.minor = element_blank(),
                          legend.position = "bottom",
                          #legend.direction = "horizontal",
                          legend.key = element_blank(),
                          legend.key.size= unit(1, "cm"),
                          #legend.margin = unit(0, "cm"),
                          legend.text = element_text(face = "bold",size = rel(1)),
                          legend.background = element_blank(),
                          #legend.title = element_text(face="italic"),
                          plot.margin=unit(c(10,5,5,5),"mm"),
                          strip.background=element_rect(colour="#f0f0f0",fill="#f0f0f0"),
                          strip.text = element_text(face="bold") )

## texreg package
## Texreg: does not (yet) support nrm or gyhpe-class
# use the extract()-function to make this available
extract.nrm.cluster <- function(model,sumsum){
  # calculate SE, tvalues and pvalues
  coeffic <- as.numeric(model$coef)
  stderr <- (model$confint[,2] - model$confint[,1])/5.15
  tvalues = coeffic/stderr
  pval <- exp(-0.717*tvalues - 0.416*tvalues^2)
  
  # then create and return a texreg object (replace NULL with actual values):
  tr <- createTexreg(
    coef.names = names(model$coef),    # character vector of coefficient labels
    coef = coeffic,          # numeric vector with coefficients
    se = stderr,            # numeric vector with standard error values
    pvalues = pval,       # numeric vector with p-values
    gof.names = c("AIC", "McFadden $pseudo-R^2$"),     # character vector with goodness-of-fit labels
    gof = c(model$AIC, model$R2)           # numeric vector of goodness-of-fit statistics
    #gof.decimal = NULL    # logical vector: GOF statistic has decimal points?
  )
  return(tr)
}
setMethod("extract", signature = className("nrm", "ghype"), 
          definition = extract.nrm.cluster)

################################################################################
################################################################################
################################################################################

## DATA PREPARATION

## function to create block matrices
block_adj <- function(labels){
  blockids <- as.numeric(plyr::mapvalues(labels, from = levels(factor(labels)), 
                                         to = c(1, numbers::Primes(length(labels) * 8))[1:(length(unique(labels)))]))
  print(unique(blockids))
  print(unique(labels))
  blocks <- blockids %*% t(blockids)
}

################################################################################
## party: party_homophily
################################################################################

block_party <- block_adj(dtMPfull$partyAbbr_re)
nrlabels_party <- c(1, numbers::Primes(length(dtMPfull$partyAbbr_re) * 8))[1:(length(unique(dtMPfull$partyAbbr_re)))]

# party homophily adj matrix
party_homophily <- ifelse(matrix(block_party %in% nrlabels_party^2, nrow = 163), exp(1), 1)
diag(party_homophily) <- 0

################################################################################
## Canton homophily: canton_homophily
################################################################################

## canton block matrix
block_canton <- block_adj(as.character(dtMPfull$CantonName))
nrlabels_canton <- c(1, numbers::Primes(length(dtMPfull$CantonName) * 8))[1:(length(unique(dtMPfull$CantonName)))]

# canton homophily adj matrix
canton_homophily <- ifelse(matrix(block_canton %in% nrlabels_canton^2, nrow = 163), exp(1), 1)


################################################################################
## Gender: gender_homophily
################################################################################

## gender block matrix
block_gender <- block_adj(dtMPfull$GenderAsString)

# gender homophily adj matrix
gender_homophily <- ifelse(matrix(block_gender %in% c(1, 4), nrow = 163), exp(1), 1)
diag(gender_homophily) <- 0

################################################################################
## Committee memberships: same_committee
################################################################################

##
same_committee <- matrix(1, nrow = nrow(cosponsfull), ncol = ncol(cosponsfull))
for(i in 1:nrow(cosponsfull)){
  for(j in 1:ncol(cosponsfull)){
    committeei <- unique(unlist(stringr::str_split(dtMPfull$committees[i], ";")))
    committeej <- unique(unlist(stringr::str_split(dtMPfull$committees[j], ";")))
    same_committee[i, j] <- length(intersect(committeei, committeej))
  }
}
diag(same_committee) <- 0
rownames(same_committee) <- rownames(cosponsfull)
colnames(same_committee) <- colnames(cosponsfull)

##
same_committee_cat <- (same_committee == 0)*(exp(1)-1) + 1
same_committee[same_committee == 0] <- 1
diag(same_committee) <- 0

################################################################################
## Age difference: age_difference
################################################################################

##
age_difference <- matrix(1, nrow = nrow(cosponsfull), ncol = ncol(cosponsfull))
for(i in 1:nrow(cosponsfull)){
  for(j in 1:ncol(cosponsfull)){
    age_difference[i, j] <- abs(dtMPfull$age[i]-dtMPfull$age[j])
  }
}
diag(age_difference) <- 0
rownames(age_difference) <- rownames(cosponsfull)
colnames(age_difference) <- colnames(cosponsfull)

##
age_difference_cat <- (age_difference == 0)*(exp(1)-1) + 1
age_difference[age_difference == 0] <- 1
diag(age_difference) <- 0

################################################################################
## Both actors participated in the both legislative periods
################################################################################

## create matrix with legislative-homophily
matSameLegis <- matrix(0, nrow = nrow(cosponsfull), ncol = ncol(cosponsfull))
for(i in 1:nrow(matSameLegis)){
  for(j in 1:ncol(matSameLegis)){
    #
    if(dtMPfull$inBothLegis[i] == dtMPfull$inBothLegis[j]){
      matSameLegis[i, j] <- 1
    }
  }
}

##
matSameLegis[matSameLegis == 1] <- exp(1)
matSameLegis[matSameLegis == 0] <- 1
diag(matSameLegis) <- 0

################################################################################
## reciprocity matrix: reciprocity
################################################################################

# reciprocity
reciprocity <- t(cosponsfull)
rownames(reciprocity) <- rownames(cosponsfull)
colnames(reciprocity) <- colnames(cosponsfull)

reciprocity_cat <- (reciprocity == 0)*(exp(1)-1) + 1
reciprocity[reciprocity == 0] <- 1
diag(reciprocity) <- 0

################################################################################
## triadic closure (calculated among the full cosponsorship network, all members form 2007-2015)
# triad_in, triad_out
################################################################################

## prepare full matrixes
triangles_ij_in <- matrix(0, nrow = nrow(cospons_nofilter), ncol = ncol(cospons_nofilter))
rownames(triangles_ij_in) <- rownames(cospons_nofilter)
colnames(triangles_ij_in) <- colnames(cospons_nofilter)
triangles_ij_out <- matrix(0, nrow = nrow(cospons_nofilter), ncol = ncol(cospons_nofilter))
rownames(triangles_ij_out) <- rownames(cospons_nofilter)
colnames(triangles_ij_out) <- colnames(cospons_nofilter)

## calculate weighted triadic closure
for(i in 1:nrow(cospons_nofilter)){
  for(j in 1:ncol(cospons_nofilter)){
    if(i != j){
      # outgoing
      sharedp_ofi <- cospons_nofilter[i,][cospons_nofilter[i,] > 0] #outgoing
      sharedp_ofj <- cospons_nofilter[j,][cospons_nofilter[j,] > 0] #outgoing
      sharedp_ij <- unique(intersect(sharedp_ofi, sharedp_ofj))
      
      for(k in sharedp_ij){
        triangles_ij_out[i,j] <- triangles_ij_out[i,j] + min(cospons_nofilter[i,k],cospons_nofilter[j,k])
      }
      # incoming
      supporter_ofi <- cospons_nofilter[,i][cospons_nofilter[,i] > 0] #incoming
      supporter_ofj <- cospons_nofilter[,j][cospons_nofilter[,j] > 0] #incoming
      sharedsup_ij <- unique(intersect(supporter_ofi, supporter_ofj))
      
      for(k in sharedsup_ij){
        triangles_ij_in[i,j] <- triangles_ij_in[i,j] + min(cospons_nofilter[k,i],cospons_nofilter[k,j])
      }
    }
  }
}

## subset triangles matrix for our reduced matrix (from 315 to 163 MPs)
triad_in <- triangles_ij_in[rownames(cosponsfull), rownames(cosponsfull)]
triad_out <- triangles_ij_out[rownames(cosponsfull), rownames(cosponsfull)]
identical(rownames(cosponsfull), rownames(triad_in))
triad_in_cat <- (triad_in == 0)*(exp(1)-1) + 1
triad_in[triad_in==0] <- 1
diag(triad_in) <- 0
triad_out_cat <- (triad_out == 0)*(exp(1)-1) + 1
triad_out[triad_out==0] <- 1
diag(triad_out) <- 0

################################################################################
################################################################################
################################################################################

## ANALYSIS

################################################################################
## Table 1: Summary table - prepared manually
################################################################################

################################################################################
## Figure 1: Heatplot
################################################################################

## sorted right to left
allparties <- c('EDU', 'SVP', 'Lega', 'FDP', 'BDP', 'CVP', 'EVP', 'glp', 'csp-ow', 'SP', 'GPS')

## compare ximat with cosponsfull => get share of sponsored bill for each dyad
dtsharecosp <- reshape2::melt(cosponsfull)
names(dtsharecosp) <- c('sender', 'target', 'cospons')
dtsharecosp$xi <- reshape2::melt(ximat)[,3]
dtsharecosp$share_cospons <- ifelse(dtsharecosp$cospons == 0, NA, ifelse(dtsharecosp$xi == 0, NA, dtsharecosp$cospons/dtsharecosp$xi))
## for each party: check average share cosponsored
dtsharecosp$partySender <- dtMPfull$partyAbbr_re[match(dtsharecosp$sender, dtMPfull$FirstLastName)]
dtsharecosp$partySender[dtsharecosp$sender == 'Christa Markwalder'] <- dtMPfull$partyAbbr_re[dtMPfull$FirstLastName == 'Christa Markwalder Baer']
dtsharecosp$partyTarget <- dtMPfull$partyAbbr_re[match(dtsharecosp$target, dtMPfull$FirstLastName)]
dtsharecosp$partyTarget[dtsharecosp$target == 'Christa Markwalder'] <- dtMPfull$partyAbbr_re[dtMPfull$FirstLastName == 'Christa Markwalder Baer']
## average/median of cosponsorship share for each party-party dyad
summary(dtsharecosp$share_cospons)
dtsharecosp_party <- dtsharecosp %>% 
  group_by(partySender, partyTarget) %>% 
  summarise(meanShare = mean(share_cospons, na.rm = TRUE), 
            meanSharePerc = round(mean(share_cospons, na.rm = TRUE)*100,0), 
            medianShare = median(share_cospons, na.rm = TRUE)
  )
dtsharecosp_party <- as.data.frame(dtsharecosp_party)
dtsharecosp_party$partySender <- factor(dtsharecosp_party$partySender, levels = allparties)
dtsharecosp_party$partyTarget <- factor(dtsharecosp_party$partyTarget, levels = allparties)
#dtsharecosp_party
# recode missings into zeros
dtsharecosp_party[is.na(dtsharecosp_party)] <- 0
# remove data for 4 parties that only have 1 member (self-loops not possible)
dtsharecosp_party <- subset(dtsharecosp_party, !(dtsharecosp_party$partySender == 'EVP' & dtsharecosp_party$partyTarget == 'EVP') & 
                              !(dtsharecosp_party$partySender == 'csp-ow' & dtsharecosp_party$partyTarget == 'csp-ow') & 
                              !(dtsharecosp_party$partySender == 'Lega' & dtsharecosp_party$partyTarget == 'Lega') & 
                              !(dtsharecosp_party$partySender == 'EDU' & dtsharecosp_party$partyTarget == 'EDU'))
## average online similarity for each dyad: non-symmetric similarity
matpartysim_politnetz_simcontent <- el %>% group_by(partySenderfull, partyTargetfull) %>%
  summarise(mean = mean(politnetz_content), 
            n = n())
matpartysim_politnetz_simcontent <- as.data.frame(matpartysim_politnetz_simcontent)
matpartysim_politnetz_simcontent$partySenderfull <- factor(matpartysim_politnetz_simcontent$partySenderfull, levels = allparties)
matpartysim_politnetz_simcontent$partyTargetfull <- factor(matpartysim_politnetz_simcontent$partyTargetfull, levels = allparties)

## heatplots: 
p1sim <- ggplot(matpartysim_politnetz_simcontent, aes(y = partySenderfull, x = partyTargetfull))+
  geom_tile(aes(fill = mean)) + ylab("") + xlab("") +
  scale_fill_gradient(
    "Avg. online support similarity",
    low = "#186318",
    high = "#3DFC3D",
    # limits = c(0,1)
  ) +
  publicationtheme +
  ggtitle('Avg. online support similarities between parties')

p2cosp <- ggplot(dtsharecosp_party, aes(y = partySender, x = partyTarget))+
  geom_tile(aes(fill = meanSharePerc/100)) +  ylab("") + xlab("") +
  scale_fill_gradient(
    "Avg. co-sponsorship support (%-share)",
    low = "#31617D",
    high = "#AEE0FC",
    #limits = c(0,1)
  ) +
  publicationtheme + 
  ggtitle('Avg. co-sponsorship support between parties')

## combine plots
p <- gridExtra::arrangeGrob(gridExtra::grid.arrange(p1sim, p2cosp, nrow = 1))
ggsave(file = 'output/Fig1_heatplot.pdf', p, width = 24, height = 14, units = 'cm')

# p <- gridExtra::arrangeGrob(gridExtra::grid.arrange(p1sim, p3cosp, nrow = 1))
# ggsave(file = 'figures/heat_onlineOfflinesupport.pdf', p, width = 18, height = 14, units = 'cm')
# 
# p1sim + coord_fixed() +
#   theme(legend.position = 'none',
#         axis.text.x = element_text(angle=90),
#         panel.background = element_blank(),
#         plot.margin = unit(c(0, 0, 0, 0), "null")) + ggtitle(element_blank()) + ylab(element_blank()) + xlab(element_blank())
# ggsave(file = 'figures/heat_onlineOfflinesupport_p1.pdf', width = 7, height = 7, units = 'cm')
# p2cosp + coord_fixed() +
#   theme(legend.position = 'none',
#         axis.text.x = element_text(angle=90),
#         panel.background = element_blank(),
#         plot.margin = unit(c(0, 0, 0, 0), "null")) + ggtitle(element_blank()) + ylab(element_blank()) + xlab(element_blank())
# ggsave(file = 'figures/heat_onlineOfflinesupport_p3.pdf', width = 7, height = 7, units = 'cm')

################################################################################
## Table 2: GHypEG Results
################################################################################

## Model 1
nfit1 <- nrm(adj = cosponsfull, 
             w = list(politnetzfull = politnetzfull), 
             directed = TRUE, selfloops = FALSE,
             xi = ximat,
             init = c(0.5))
AIC(nfit1)

## Model 2
nfit2 <- nrm(adj = cosponsfull, 
             w = list(politnetz_content = politnetz_content,
                      eps_politnetz_content = eps_politnetz_content,
                      # Controls
                      politnetz_activity = politnetz_absdiffactivity,
                      politnetz_activity_diff0 = politnetz_absdiffactivity_diff0), 
             directed = TRUE, selfloops = FALSE, regular = FALSE,
             xi = ximat,
             init = c(0.5, .1))
AIC(nfit2)

## Model 3
nfit3 <- nrm(adj = cosponsfull, 
             w = list(politnetz_content = politnetz_content,
                      eps_politnetz_content = eps_politnetz_content,
                      # Controls
                      recip = reciprocity,
                      recip_cat = reciprocity_cat,
                      triads_in = triad_in,
                      triads_in_cat = triad_in_cat,
                      triads_out = triad_out,
                      triads_out_cat = triad_out_cat,
                      committee_count = same_committee,
                      committee = same_committee_cat,
                      gender = gender_homophily,
                      age = age_difference,
                      age_diff0 = age_difference_cat,
                      party = party_homophily,
                      canton = canton_homophily,
                      politnetz_activity = politnetz_absdiffactivity,
                      politnetz_activity_diff0 = politnetz_absdiffactivity_diff0),
             xi = ximat,
             directed = TRUE, selfloops = FALSE,
             init = c(0.6, 0.06, 0.44, 0.19, 0.26, 0.03, 0.12, -0.05, -.54, 0.17, 0))
AIC(nfit3)

## Model 4
nfit4 <- nrm(adj = cosponsfull, 
             w = list(politnetz_content = politnetz_content,
                      eps_politnetz_content = eps_politnetz_content,
                      # Controls
                      recip = reciprocity,
                      recip_cat = reciprocity_cat,
                      party = party_homophily,
                      politnetz_activity = politnetz_absdiffactivity,
                      politnetz_activity_diff0 = politnetz_absdiffactivity_diff0
             ), xi = ximat,
             directed = TRUE, selfloops = FALSE,
             init = c(0.02, 0.02, 0.5, 0.02))

## Latex table: 
texreg(list(nfit1, nfit2, nfit3, nfit4), digits = 3, 
       custom.coef.names = c('Direct online support', 
                             'Online support similarity',
                             'Online support similarity - Missing Data',
                             'Online support activity',
                             'Online support activity - diff == 0',
                             'Reciprocity (weighted)',
                             'Reciprocity (categorical)',
                             'Shared partners (incoming, weighted)',
                             'Shared partners (incoming, categorical)',
                             'Shared partners (outgoing, weighted)',
                             'Shared partners (outgoing, categorical)',
                             'Shared committees (weighted)',
                             'Shared committees (categorical)',
                             'Gender homophily', 
                             'Age difference',
                             'Age difference == 0',
                             'Party homophily',
                             'State (=Canton) homophily'
       ),
       dcolumn = TRUE, booktabs = TRUE, single.row = TRUE,
       reorder.coef = c(1:2, 6,8,10,12,14,15,17,18, 4, 3,5,7,9,11,13,16),
       groups = list('Online behavior' = 1:2, 'Endogenous variables' = 3:6,
                     'Homophily' = 7:10, 'Control' = 11, 'Correcting factors' = 12:18),  
       caption = 'Hypernets regression output', label = 'hypernetsreg', 
       custom.model.names = c('(1)', '(2)', '(3)', '(4)'), 
       file = 'output/Table2_ghypEG.tex')


################################################################################
## Figure 2: Marginal Effects Plot
################################################################################

## get omega values
nfit2omega <- data.frame(omega = nfit2$omega, 
                         cosponsfull = c(cosponsfull), 
                         politnetzsim = c(politnetz_content), 
                         sameparty = c(party_homophily),
                         no_data = as.vector(nfit2$predictors$eps_politnetz_content)!=1) %>%
  filter(!no_data)
nfit2omega <- na.omit(nfit2omega)
nfit3omega <- data.frame(omega = nfit3$omega, 
                         cosponsfull = c(cosponsfull), 
                         politnetzsim = c(politnetz_content), 
                         sameparty = c(party_homophily),
                         no_data = as.vector(nfit3$predictors$eps_politnetz_content)!=1) %>%
  filter(!no_data)
nfit3omega <- na.omit(nfit3omega)

## rescale omega
nfit3omega$omega <- nfit3omega$omega/max(nfit3omega$omega)

## plot
p1 <- ggplot(nfit2omega, aes(x = politnetzsim, y = omega, color = factor(sameparty)))+
  geom_point(alpha = .1) +
  #coord_cartesian(ylim = c(0, 1)) +
  geom_smooth() + publicationtheme +
  scale_color_manual("", values = c('#E41A1C', '#377EB8'), labels = c('Between parties', 'Within party')) +
  xlab("Politnetz support similarity") + ylab("Pr(Cosponsorship)") +
  ggtitle('Model (2): Marginal effects of online support similarity')

p2 <- ggplot(nfit3omega, aes(x = politnetzsim, y = omega, color = factor(sameparty)))+
  geom_point(alpha = .1) +
  geom_smooth() + publicationtheme +
  scale_color_manual("", values = c('#E41A1C', '#377EB8'), labels = c('Between parties', 'Within party'))+
  xlab("Politnetz support similarity") + ylab("Pr(Cosponsorship)")+
  ggtitle('Model (3): Marginal effects of online support similarity')
p <- gridExtra::arrangeGrob(gridExtra::grid.arrange(p1, p2, nrow = 1))
ggsave(file = 'output/Fig2_ME.pdf', p, width = 28, height = 14, units = 'cm')

################################################################################
################################################################################
################################################################################

## SUPPLEMENTARY INFORMATION ONLINE

################################################################################
## SI Online, Figure 2
################################################################################

p1 <- ggplot(el, aes(x = politnetz_absdiffactivity, y = politnetz_content))+#, color = factor(partyHom)
  geom_point(alpha = .2, color = 'grey')+
  geom_smooth(aes(color = 'Smoothed line')) +
  publicationtheme + 
  ylab("Similar online support (0 = dissimilar, 1 = similar)") +
  xlab("Difference in support activity (nr of supportees named)") +
  scale_color_manual("", labels = c('Smoothed line'), values = c('blue'))+
  geom_hline(yintercept = .5) + geom_vline(xintercept = 150)

p2 <- ggplot(el, aes(x = politnetz_absdiffactivity, y = politnetz_content, color = factor(partyHom)))+
  geom_point(alpha = .2, color = 'grey')+
  geom_smooth() +
  publicationtheme + 
  ylab("Similar online support (0 = dissimilar, 1 = similar)") +
  xlab("Difference in support activity (nr of supportees named)") +
  scale_color_manual("", values = c('forestgreen', 'firebrick'), labels = c('Across parties', 'Within party'))

## combine ggplots
p <- gridExtra::arrangeGrob(gridExtra::grid.arrange(p1, p2, nrow = 1))
ggsave(file = 'output/SIOnline_Fig2.pdf', p, width = 24, height = 12, units = 'cm')

################################################################################
## SI Online, Table 1
################################################################################

## Model 1
nfit1si <- nrm(adj = cosponsfull, 
             w = list(politnetzfull = politnetzfull), 
             directed = TRUE, selfloops = FALSE,
             xi = ximat,
             init = c(0.5))

## Model 2
nfit2si <- nrm(adj = cosponsfull, 
             w = list(politnetz = politnetzsimfull,
                      eps_politnetz = eps_politnetzsimfull), 
             directed = TRUE, selfloops = FALSE,
             xi = ximat,
             init = c(0.4, 0.0))

## Model 3
nfit3si <- nrm(adj = cosponsfull, 
             w = list(politnetz = politnetzsimfull,
                      eps_politnetz = eps_politnetzsimfull,
                      # Control
                      recip = reciprocity,
                      recip_cat = reciprocity_cat,
                      triads_in = triad_in,
                      triads_in_cat = triad_in_cat,
                      triads_out = triad_out,
                      triads_out_cat = triad_out_cat,
                      committee_count = same_committee,
                      committee = same_committee_cat,
                      gender = gender_homophily,
                      age = age_difference,
                      age_diff0 = age_difference_cat,
                      party = party_homophily,
                      canton = canton_homophily  ), 
             directed = TRUE, selfloops = FALSE, xi = ximat,
             init = c(0.06, 0.44, 0.19, 0.26, 0.03, 0.12, -0.05, -.54, 0.17))

## Model 4
nfit4si <- nrm(adj = cosponsfull, 
             w = list(politnetz = politnetzsimfull,
                      eps_politnetz = eps_politnetzsimfull,
                      # Control
                      recip = reciprocity,
                      recip_cat = reciprocity_cat,
                      party = party_homophily 
             ), 
             directed = TRUE, selfloops = FALSE,
             xi = ximat,
             init = c(0.02, 0.0, 0.5, 0.1, 0.5))

##
texreg(list(nfit1si, nfit2si, nfit3si, nfit4si), digits = 3, 
       custom.coef.names = c('Direct online support', 
                             'Online support similarity',
                             'Online support similarity - Missing Data',
                             'Reciprocity (weighted)',
                             'Reciprocity (categorical)',
                             'Shared partners (incoming, weighted)',
                             'Shared partners (incoming, categorical)',
                             'Shared partners (outgoing, weighted)',
                             'Shared partners (outgoing, categorical)',
                             'Shared committees (weighted)',
                             'Shared committees (categorical)',
                             'Gender homophily', 
                             'Age difference',
                             'Age difference == 0',
                             'Party homophily',
                             'State (=Canton) homophily'
       ), 
       dcolumn = TRUE, booktabs = TRUE, single.row = TRUE,
       reorder.coef = c(1:2, 4,6,8,10,12,13,15,16, 3,5,7,9,11,14),
       groups = list('Online behavior' = 1:2, 'Endogenous variables' = 3:6,
                     'Homophily' = 7:10, 'Correcting factors' = 12:16), 
       caption = 'Hypernets regression output', label = 'hypernetsreg', 
       custom.model.names = c('(1)', '(2)', '(3)', '(4)'), 
       file = 'output/SIOnline_Table1.tex')

################################################################################
## SI Online, Figure 3
################################################################################

## variable names
varnames <- c(#'Direct online support', 
  'Online support similarity',
  'Online support similarity - Missing Data',
  'Reciprocity (weighted)',
  'Reciprocity (categorical)',
  'Shared partners (incoming, weighted)',
  'Shared partners (incoming, categorical)',
  'Shared partners (outgoing, weighted)',
  'Shared partners (outgoing, categorical)',
  'Shared committees (weighted)',
  'Shared committees (categorical)',
  'Gender homophily', 
  'Age difference',
  'Age difference == 0',
  'Party homophily',
  'State (=Canton) homophily',
  'Online support activity',
  'Online support activity - diff == 0')

##
permutinitial <-  coef(nfit3)
permutinitial[1:2] <- 0
ix <- mat2vec.ix(politnetz_content, directed = TRUE, selfloops = FALSE) 
ix_data <- eps_politnetz_content[ix] == 1
##
# pb <- txtProgressBar(0, 100, style = 3)
# for(i in 1:100){
# setTxtProgressBar(pb, i)
permute_nrm <- function(i){
  ## permuate politnetz
  polnetzsim_Vect_ix <- rep(1, sum(ix))
  polnetzsim_Vect_ix[ix_data] <- sample(politnetz_content[ix][ix_data]) #replace = FALSE, default
  politnetzsimfull_shuffled <- vec2mat(polnetzsim_Vect_ix, directed = TRUE, 
                                       selfloops = FALSE, n=163)
  ## estimate the nrm
  tempfit <- nrm(adj = cosponsfull, 
                 w = list(     politnetz_content = politnetzsimfull_shuffled,
                               eps_politnetz_content = eps_politnetz_content,
                               recip = reciprocity,
                               recip_cat = reciprocity_cat,
                               triads_in = triad_in,
                               triads_in_cat = triad_in_cat,
                               triads_out = triad_out,
                               triads_out_cat = triad_out_cat,
                               committee_count = same_committee,
                               committee = same_committee_cat,
                               gender = gender_homophily,
                               age = age_difference,
                               age_diff0 = age_difference_cat,
                               party = party_homophily,
                               canton = canton_homophily,
                               politnetz_activity = politnetz_absdiffactivity,
                               politnetz_activity_diff0 = politnetz_absdiffactivity_diff0
                 ), xi = ximat, directed = TRUE, selfloops = FALSE,
                 init = permutinitial, ci = FALSE)
  
  ## save coefficients
  dtflex <- #rbind(dtflex, 
    data.frame(coef = tempfit$coef, type = 'random', 
               coefname = varnames)
  return(dtflex)
  # )
}
set.seed(123)
dtflex_list <- pbmcapply::pbmclapply(FUN = permute_nrm, X = 1:100, mc.cores = 10)
dtflex <- data.frame(coef = nfit3$coef, type = 'original',
                     coefname = varnames)[c(1,3,5,7,9,11:12,14:16),]
for(out in dtflex_list){
  dtflex <- rbind(dtflex, 
                  out[c(1,3,5,7,9,11:12,14:16),])
}
dtflex$coefname <- factor(dtflex$coefname, levels = varnames[sort(c(1,3,5,7,9,11:12,14:16), decreasing = TRUE)], ordered = TRUE)

## calculate mean and CI
dtflex_meanci <- as.data.frame(dtflex[dtflex$type == 'random',] %>%
                                 group_by(coefname) %>%
                                 summarise(mean = mean(coef), 
                                           lowCI = t.test(coef)$conf.int[[1]], 
                                           highCI = t.test(coef)$conf.int[[2]]))

## plot it
ggplot()+
  geom_point(data = dtflex_meanci, aes(x = coefname, y = mean, color = 'random'))+
  geom_point(data = dtflex[dtflex$type == 'random' & dtflex$coefname == 'Online support similarity',], 
             aes(x = coefname, y = coef, color = 'grey'), alpha = .2)+
  geom_point(data = dtflex[dtflex$type == 'original',], 
             aes(x = coefname, y = coef, group = coefname, color = 'original'))+
  geom_errorbar(data = dtflex_meanci, aes(x = coefname, ymin = lowCI, ymax = highCI, color = 'random'))+
  coord_flip() +
  scale_color_manual("", values = c('grey40', '#E41A1C', '#377EB8'), labels = c('permutation', 'original coefficient', 'permutation coefficient'))+
  publicationtheme +
  xlab("")+ylab("Coefficient size (+CI)")
ggsave(file = "output/SIOnline_Fig3.pdf", 
       width = 24, height = 12, units = 'cm')

################################################################################
## SI Online, Figure 4
################################################################################

############ Homophily: gender, canton, committee
el$sameCom_dummy <- ifelse(el$sameCommittee > 1, exp(1), 1)
mel <- reshape::melt(subset(el, select = c('politnetz_content', 'genderHom', 
                                           'cantonHom', 'sameCom_dummy', 'partyHom')),
                     id = c('politnetz_content', 'partyHom'))
## rename some stuff
mel$variable_named <- NA
mel$variable_named[mel$variable == 'genderHom'] <- 'Gender homophily'
mel$variable_named[mel$variable == 'cantonHom'] <- 'Cantonal homophily'
mel$variable_named[mel$variable == 'sameCom_dummy'] <- 'Shared committee(s)'
mel$partyHom_named <- NA
mel$partyHom_named[mel$partyHom == 1] <- 'Between parties'
mel$partyHom_named[mel$partyHom != 1] <- 'Within party'

##
p1 <- ggplot(mel, aes(y = politnetz_content, x = factor(variable_named), color = factor(value)))+
  geom_boxplot()+
  publicationtheme + facet_wrap(~partyHom_named, nrow = 2) +
  ylab("Similar online support (0 = dissimilar, 1 = similar)") +
  xlab("") +
  ggtitle("All dyads (N=26,406)") +
  scale_color_manual("", values = c('forestgreen', 'skyblue2'), labels = c('Heterophily', 'Homophily'))
p2 <- ggplot(subset(mel, mel$politnetz_content>=.5), aes(y = politnetz_content, x = factor(variable_named), color = factor(value)))+
  geom_boxplot()+
  coord_cartesian(ylim = c(0.5, 1)) + 
  publicationtheme + facet_wrap(~partyHom_named, nrow = 2) +
  ylab("Similar online support (0 = dissimilar, 1 = similar)") + 
  xlab("") +
  ggtitle("Dyads with high online similarity (N=1,665)") +
  scale_color_manual("", values = c('forestgreen', 'skyblue2'), labels = c('Heterophily', 'Homophily'))

## combine ggplots
p <- gridExtra::arrangeGrob(gridExtra::grid.arrange(p1, p2, nrow = 1))
ggsave(file = 'output/SIOnline_Fig4.pdf', p, width = 28, height = 18, units = 'cm')

################################################################################
## SI Online, Figure 5
################################################################################

############ Age difference
p1 <- ggplot(el, aes(y = politnetz_content, x = agediff, color = factor(partyHom)))+
  geom_point(alpha = .2, color = 'grey') +
  geom_smooth()+
  publicationtheme +
  coord_cartesian(ylim = c(0, 1)) + 
  ylab("Online Support Similarity") + 
  xlab("|∆(Age)|") +
  ggtitle("All dyads (N=26,406)") +
  scale_color_manual("", values = c('#E41A1C', '#377EB8'), labels = c('Between parties', 'Within party'))
p2 <- ggplot(subset(el, el$politnetz_content >= .5), aes(y = politnetz_content, x = agediff, color = factor(partyHom)))+
  geom_point(alpha = .2, color = 'grey') +
  geom_smooth()+
  publicationtheme +
  coord_cartesian(ylim = c(0.5, 1)) + 
  ylab("Online Support Similarity") + 
  xlab("|∆(Age)|") +
  ggtitle("Dyads with high online similarity (N=1,665)") +
  scale_color_manual("", values = c('#E41A1C', '#377EB8'), labels = c('Between parties', 'Within party'))
# combine plots
p <- gridExtra::arrangeGrob(gridExtra::grid.arrange(p1, p2, nrow = 1))
ggsave(file = 'output/SIOnline_Fig5.pdf', p, width = 24, height = 12, units = 'cm')

# p1 + #coord_fixed() + 
#   theme(legend.position = 'none', 
#         panel.background = element_blank(),
#         plot.margin = unit(c(0.25, 0.25, 0.25, 0.25), "cm")) + ggtitle(element_blank()) + ylab('OSS') + xlab('|Delta Age|')
# ggsave(file = '2_Data_Analysis/output/cor_onlineSim_Agediff_p1.pdf', width = 9, height = 7, units = 'cm')
# p2 + #coord_fixed() + 
#   theme(legend.position = 'none', 
#         panel.background = element_blank(),
#         plot.margin = unit(c(0.25, 0.25, 0.25, 0.25), "cm")) + ggtitle(element_blank()) + ylab('OSS') + xlab('|Delta Age|')
# ggsave(file = '2_Data_Analysis/output/cor_onlineSim_Agediff_p2.pdf', width = 9, height = 7, units = 'cm')

# check correlation
cor.test(el$politnetz_content, el$agediff) 
# data:  el$politnetz_content and el$agediff
# t = -0.90005, df = 26404, p-value = 0.3681
# alternative hypothesis: true correlation is not equal to 0
# 95 percent confidence interval:
#   -0.017599242  0.006522966
# sample estimates:
#   cor 
# -0.005538944
############ 

################################################################################
## SI Online, Table 2
################################################################################

##
print(xtable::xtable(data.frame(dtMPfull$FirstName, dtMPfull$LastName, 
                                dtMPfull$CantonAbbreviation, dtMPfull$PartyAbbreviation, 
                                ifelse(isTRUE(dtMPfull$inBothLegis), 'yes', 'no'))), 
      include.rownames = FALSE, file = 'output/SIOnline_Table2.tex')

################################################################################
## SI Online, Table 3
################################################################################

# List of parties
partydt <- as.data.frame(dtMPfull %>%
                           group_by(dtMPfull$PartyName) %>%
                           summarize(abbreviation = unique(PartyAbbreviation),
                                     recoded = unique(partyAbbr_re),
                                     n = n(),
                                     leftright = unique(party_leftright), 
                                     econ = unique(party_econlr), 
                                     libertrad = unique(party_libtrad)
                           ))
print(xtable::xtable(partydt), 
      include.rownames = FALSE, file = 'output/SIOnline_Table3.tex')

################################################################################
################################################################################
################################################################################